package seminar

import scala.collection.mutable.Map

abstract class Expr {
  def value(map: Map[String, Int]): Int
}

case class Number(i: Int) extends Expr {
  def value(map: Map[String, Int]) = i
}

case class Variable(name: String) extends Expr {
  def value(map: Map[String, Int]) = map(name)
}

abstract class BinExpr(expr1: Expr, expr2: Expr, op: (Int, Int) => Int) extends Expr {
  def value(map: Map[String, Int]) = op(expr1.value(map), expr2.value(map))
}

case class Add(expr1: Expr, expr2: Expr) extends BinExpr(expr1, expr2, _ + _)
case class Sub(expr1: Expr, expr2: Expr) extends BinExpr(expr1, expr2, _ - _)
case class Mul(expr1: Expr, expr2: Expr) extends BinExpr(expr1, expr2, _ * _)
case class Div(expr1: Expr, expr2: Expr) extends BinExpr(expr1, expr2, _ / _) 