package seminar

trait Parser {

  def newParser[T](f: String => Result[T]) = new Parser[T] {
    def apply(string: String) = f(string)
  }
  implicit def accept(required: String) = newParser(string =>
    if (string.startsWith(required)) Success(required, string.substring(required.length()))
    else Failure()
  ) token

  abstract class Parser[T] extends (String => Result[T]) {

    def acceptIf(predicate: T => Boolean) = newParser(string =>
      this(string) match {
        case Success(t, string1) if predicate(t) => Success(t, string1)
        case _ => Failure[T]
      }
    )

    def |(parser: Parser[T]) = newParser(string =>
      this(string) match {
        case Failure() => parser(string)
        case success => success
      }
    )

    def ~[U](parser: => Parser[U]) = newParser(string =>
      this(string) match {
        case Success(t, string1) => parser(string1) match {
          case Success(u, string2) => Success((t, u), string2)
          case Failure() => Failure[(T, U)]
        }
        case Failure() => Failure[(T, U)]
      }
    )
    def map[U](f: T => U) = newParser(string => this(string).map(f))
    def ~>[U](parser: => Parser[U]) = this ~ parser map (_._2)
    def <~[U](parser: => Parser[U]) = this ~ parser map (_._1)
    def token = this <~ (space *)
    def &>[U](k: T => Parser[U]) = newParser(string =>
      this(string) match {
        case Failure() => Failure[U]
        case Success(t, string1) => k(t)(string1)
      }
    )
    def * : Parser[List[T]] =
      ((this ~ (this*)) map (pair => pair._1 :: pair._2)) |
        result(List())
  }

  def char: Parser[Char] = newParser(string =>
    if (string.length > 0) Success(string.head, string.tail)
    else Failure()
  )

  def failure[T] = newParser(_ => Failure[T]())
  def result[T](t: T) = newParser(Success(t, _))
  def digit = char acceptIf (_.isDigit)
  def letter = char acceptIf (_.isLetter)
  def space = char acceptIf (_.isSpaceChar)
  def alphanum = letter | digit
  def letters = (letter *) map (_.mkString)

  def word = letters acceptIf (_.nonEmpty) token
  def integer1 = (digit*)
  def digitVal = digit map (d => d - 48)
  def buildInteger(n: Int)(d: Int) = 10 * n + d
  def integerp(n: Int): Parser[Int] = digitVal.map(buildInteger(n)) &> integerp | result(n)
  def integer = digitVal &> integerp token
}