package seminar
import scala.collection.mutable.Map

sealed abstract class Statement {
  def run(memory: Map[String, Int])
}
case class Assignment(name: String, expr: Expr) extends Statement {
  def run(memory: Map[String, Int]) = memory += (name -> expr.value(memory))
}
case class Skip() extends Statement {
  def run(memory: Map[String, Int]) = ()
}

case class While(expr: Expr, statement: Statement) extends Statement {
  def run(memory: Map[String, Int]) = if (expr.value(memory) > 0) {
    statement.run(memory)
    run(memory)
  }
}
case class If(expr: Expr, statement1: Statement, statement2: Statement) extends Statement {
  def run(memory: Map[String, Int]) = if (expr.value(memory) > 0) {
    statement1.run(memory)
  } else {
    statement2.run(memory)
  }
}
case class Read(name: String) extends Statement {
  def run(memory: Map[String, Int]) = {
    println(name + "= ")
    memory += (name -> readInt())
  }
}
case class Write(expr: Expr) extends Statement {
  def run(memory: Map[String, Int]) = println(expr.value(memory))
}
case class Block(statements: List[Statement]) extends Statement {
  def run(memory: Map[String, Int]) = for (statement <- statements) statement.run(memory)
}
